#!/usr/bin/env python
# -*- coding: iso-8859-1 -*-



################################################
#############   MACHINE LEARNING   #############
##############     -  2016 -    ################
################################################

"""
Contact: Adrian Ulges (adrian.ulges@hs-rm.de)
"""


from preprocess_documents import read_and_preprocess_documents
import sys
import argparse
import os

"""
  Text Classification
  -------------------------
  an interface for English document classification.
  Train the classifier on the training documents like this:

  > python classifier.py --train nytimes_data/train/*/*

  Apply the classifier to the test documents like this:

  > python classifier.py --apply nytimes_data/test/*/*
"""

class Classifier:

    def __init__(self):
        pass

    def train(self, features, labels):
        """
        trains a document classifier and stores the trained model
        (+ other relevant information).

        @type features: dict
        @param features: Each entry in 'features' represents a document
                         by its (sparse) bag-of-words vector. 'features'
                         is of the following form (i.e., for each document, 
                         all terms occurring in the document and their
                         counts are stored in a dictionary):
                         {
                           'doc1.txt':
                              {
                                'the' : 17,
                                'world': 3, 
                                ...
                              },
                           'doc2.txt':
                              {
                                'community' : 2,
                                'college': 1, 
                                ...
                              },
                            ...
                         }
        @type labels: dict
        @param labels: 'labels' contains the class labels for all documents
                       in dictionary form:
                       {
                           'doc1.txt': 'arts',
                           'doc2.txt': 'business',
                           'doc3.txt': 'sports',
                           ...
                       }
        """
        raise NotImplementedError()


    def apply(self, features):
        """
        applies a classifier to a set of documents. Requires the classifier
        to be trained (i.e., you need to call train() before you can call test()).

        @type features: dict
        @param features: Each entry in 'features' represents a document
                         by its (sparse) bag-of-words vector. 'features'
                         is of the following form (i.e., for each document, 
                         all terms occurring in the document and their
                         counts are stored):
                         {
                           'doc1.txt':
                              {
                                'the' : 17,
                                'world': 3, 
                                ...
                              },
                           'doc2.txt':
                              {
                                'community' : 2,
                                'college': 1, 
                                ...
                              },
                            ...
                         }
        @rtype: dict
        @return: For each document classified, apply() returns the label
                 of the class the document has been assigned to. The return value
                 is a dictionary of the form:
                 {
                   'doc1.txt': 'arts',
                   'doc2.txt': 'travel',
                   'doc3.txt': 'sports',
                   ...
                 }
        """
        raise NotImplementedError()



if __name__ == "__main__":

    # parse command line arguments (no need to touch)
    parser = argparse.ArgumentParser(description='A text classifier based on Logistic Regression.')
    parser.add_argument('documents', metavar='doc', type=str, nargs='+',
                        help='documents to train/apply the classifier on/to')
    parser.add_argument('--train', help="train the classifier", action='store_true')
    parser.add_argument('--apply', help="apply the classifier (you'll need to train or load"\
                                        " a trained model before)", action='store_true')
    # for inspecting the classifier's weights (see exercise sheet)
    parser.add_argument('--inspect', help="get some info about the learned model", 
                        action='store_true')
    args = parser.parse_args()

    # reads and preprocesses the documents listed as commandline arguments. 
    # You can use the resulting features for classification.
    wordmap, features = read_and_preprocess_documents(args.documents)

    # estimate class labels ('arts', 'business', 'dining', ...)
    # from directory names
    labels = {}
    for filename in features.keys():
        # nytimes_data/train/sports/foo.html -> ['nytimes_data', 'train', 'sports', 'foo.html']
        tokens = filename.split(os.path.sep) 
        # ['nytimes_data', 'train', 'sports', 'foo.html'] -> 'sports'
        classlabel = tokens[-2]
        labels[filename] = classlabel

    # FIXME: implement your own classifier.
    classifier = Classifier()

    #  train classifier on 'features' and 'labels' 
    # (using documents from the 'train' folder)
    if args.train:
        classifier.train(features, labels)

    # apply the classifier to documents from
    # the 'test' folder
    if args.apply:
        result = classifier.apply(features)

    # FIXME: measure error rate, print misclassified docs.
